# coding: utf-8

"""
    imandra-http-server

    Query Imandra via HTTP. See also https://github.com/aestheticIntegration/bs-imandra-client for a sample client implementation and OCaml API types.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import unittest
import datetime

import imandra_http_api_client
from imandra_http_api_client.models.instance_result import InstanceResult  # noqa: E501
from imandra_http_api_client.rest import ApiException

class TestInstanceResult(unittest.TestCase):
    """InstanceResult unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testInstanceResult(self):
        """Test InstanceResult"""
        # inst = InstanceResult()

if __name__ == '__main__':
    unittest.main()
