# coding: utf-8

"""
    imandra-http-server

    Query Imandra via HTTP. See also https://github.com/aestheticIntegration/bs-imandra-client for a sample client implementation and OCaml API types.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class MethodType(str, Enum):
    """
    MethodType
    """

    """
    allowed enum values
    """
    UNROLL = 'unroll'
    EXT_SOLVER = 'ext_solver'
    AUTO = 'auto'
    INDUCT = 'induct'

    @classmethod
    def from_json(cls, json_str: str) -> MethodType:
        """Create an instance of MethodType from a JSON string"""
        return MethodType(json.loads(json_str))


