# coding: utf-8

# flake8: noqa
"""
    imandra-http-server

    Query Imandra via HTTP. See also https://github.com/aestheticIntegration/bs-imandra-client for a sample client implementation and OCaml API types.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from imandra_http_api_client.models.decompose_region import DecomposeRegion
from imandra_http_api_client.models.decompose_request_src import DecomposeRequestSrc
from imandra_http_api_client.models.decompose_result import DecomposeResult
from imandra_http_api_client.models.eval_request_src import EvalRequestSrc
from imandra_http_api_client.models.eval_response import EvalResponse
from imandra_http_api_client.models.hints import Hints
from imandra_http_api_client.models.induct_functional_body import InductFunctionalBody
from imandra_http_api_client.models.induct_structural_body import InductStructuralBody
from imandra_http_api_client.models.induct_structural_style import InductStructuralStyle
from imandra_http_api_client.models.induct_type import InductType
from imandra_http_api_client.models.instance_request_name import InstanceRequestName
from imandra_http_api_client.models.instance_request_src import InstanceRequestSrc
from imandra_http_api_client.models.instance_response import InstanceResponse
from imandra_http_api_client.models.instance_result import InstanceResult
from imandra_http_api_client.models.instance_type import InstanceType
from imandra_http_api_client.models.method import Method
from imandra_http_api_client.models.method_body import MethodBody
from imandra_http_api_client.models.method_ext_solver_body import MethodExtSolverBody
from imandra_http_api_client.models.method_induct_body import MethodInductBody
from imandra_http_api_client.models.method_induct_body_body import MethodInductBodyBody
from imandra_http_api_client.models.method_type import MethodType
from imandra_http_api_client.models.method_unroll_body import MethodUnrollBody
from imandra_http_api_client.models.printer_details import PrinterDetails
from imandra_http_api_client.models.response_error import ResponseError
from imandra_http_api_client.models.response_instance import ResponseInstance
from imandra_http_api_client.models.response_model import ResponseModel
from imandra_http_api_client.models.syntax import Syntax
from imandra_http_api_client.models.up_to_body import UpToBody
from imandra_http_api_client.models.verify_request_name import VerifyRequestName
from imandra_http_api_client.models.verify_request_src import VerifyRequestSrc
from imandra_http_api_client.models.verify_response import VerifyResponse
from imandra_http_api_client.models.verify_response_body import VerifyResponseBody
from imandra_http_api_client.models.verify_result import VerifyResult
from imandra_http_api_client.models.with_instance_body import WithInstanceBody
from imandra_http_api_client.models.with_unknown_reason_body import WithUnknownReasonBody
