# coding: utf-8

"""
    imandra-http-server

    Query Imandra via HTTP. See also https://github.com/aestheticIntegration/bs-imandra-client for a sample client implementation and OCaml API types.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import unittest

import imandra_http_api_client
from imandra_http_api_client.api.default_api import DefaultApi  # noqa: E501
from imandra_http_api_client.rest import ApiException


class TestDefaultApi(unittest.TestCase):
    """DefaultApi unit test stubs"""

    def setUp(self):
        self.api = imandra_http_api_client.api.default_api.DefaultApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_eval(self):
        """Test case for eval

        Evaluate a string of source code  # noqa: E501
        """
        pass

    def test_get_history(self):
        """Test case for get_history

        Get a text version of the Logic History  # noqa: E501
        """
        pass

    def test_get_status(self):
        """Test case for get_status

        Get server status  # noqa: E501
        """
        pass

    def test_instance_by_name(self):
        """Test case for instance_by_name

        Search for an instance from a property (boolean function) by name. (The name should exist from a previous call to `/eval`).  This is the more 'primitive' call - calls to /*/by-src are for convenience and use Imandra_util.gensym to assign a name to your `src` block first, and then act by_name, so give this endpoint a try if you're having issues with /*/by-src.   # noqa: E501
        """
        pass

    def test_instance_by_src(self):
        """Test case for instance_by_src

        Obtain an instance from a property (boolean function) from lambda expression source  # noqa: E501
        """
        pass

    def test_reset(self):
        """Test case for reset

        Reset Imandra internal state  # noqa: E501
        """
        pass

    def test_shutdown(self):
        """Test case for shutdown

        Shutdown the server  # noqa: E501
        """
        pass

    def test_verify_by_name(self):
        """Test case for verify_by_name

        Verify a property (boolean function) by name. (The name should exist from a previous call to `/eval`).  This is the more 'primitive' call - calls to /*/by-src are for convenience and use Imandra_util.gensym to assign a name to your `src` block first, and then act by_name, so give this endpoint a try if you're having issues with /*/by-src.   # noqa: E501
        """
        pass

    def test_verify_by_src(self):
        """Test case for verify_by_src

        Verify a property (boolean function) from lambda expression source  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()
